# Generated by Django 3.2.15 on 2022-10-24 11:10

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("caluma_analytics", "0004_simple_history"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="analyticsfield",
            options={"ordering": ["sort", "-created_at"]},
        ),
        migrations.AddField(
            model_name="analyticsfield",
            name="sort",
            field=models.PositiveIntegerField(db_index=True, default=0, editable=False),
        ),
        migrations.AddField(
            model_name="historicalanalyticsfield",
            name="sort",
            field=models.PositiveIntegerField(db_index=True, default=0, editable=False),
        ),
    ]
