# Generated by Django 3.2.13 on 2022-06-09 11:26

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("caluma_form", "0044_migrate_format_validators"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="historicalanswer",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical answer",
                "verbose_name_plural": "historical answers",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalanswerdocument",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical answer document",
                "verbose_name_plural": "historical answer documents",
            },
        ),
        migrations.AlterModelOptions(
            name="historicaldocument",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical document",
                "verbose_name_plural": "historical documents",
            },
        ),
        migrations.AlterModelOptions(
            name="historicaldynamicoption",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical dynamic option",
                "verbose_name_plural": "historical dynamic options",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalfile",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical file",
                "verbose_name_plural": "historical files",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalform",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical form",
                "verbose_name_plural": "historical forms",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalformquestion",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical form question",
                "verbose_name_plural": "historical form questions",
            },
        ),
        migrations.AlterModelOptions(
            name="historicaloption",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical option",
                "verbose_name_plural": "historical options",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalquestion",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical question",
                "verbose_name_plural": "historical questions",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalquestionoption",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical question option",
                "verbose_name_plural": "historical question options",
            },
        ),
        migrations.AlterField(
            model_name="historicalanswer",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicalanswerdocument",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicaldocument",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicaldynamicoption",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicalfile",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicalform",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicalformquestion",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicaloption",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicalquestion",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicalquestionoption",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
    ]
