# -*- coding: utf-8 -*-
# Generated by Django 1.11.18 on 2019-02-08 13:02
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("caluma_workflow", "0007_auto_20190206_0854")]

    operations = [
        migrations.RemoveField(model_name="workflow", name="start"),
        migrations.AddField(
            model_name="workflow",
            name="start_tasks",
            field=models.ManyToManyField(
                help_text="Starting task(s) of the workflow.",
                related_name="_workflow_start_tasks_+",
                to="caluma_workflow.Task",
            ),
        ),
    ]
