# -*- coding: utf-8 -*-
# Generated by Django 1.11.20 on 2019-05-06 06:55
from __future__ import unicode_literals

import django.contrib.postgres.fields
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("caluma_form", "0014_auto_20190429_0910")]

    operations = [
        migrations.AddField(
            model_name="question",
            name="format_validators",
            field=django.contrib.postgres.fields.ArrayField(
                base_field=models.CharField(max_length=255),
                blank=True,
                default=list,
                size=None,
            ),
        )
    ]
