# Generated by Django 3.2.12 on 2022-03-31 14:24

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("caluma_analytics", "0002_analytics_pivot_table"),
    ]

    operations = [
        migrations.AlterField(
            model_name="analyticstable",
            name="starting_object",
            field=models.CharField(
                choices=[
                    ("cases", "Cases"),
                    ("work_items", "Work Items"),
                    ("documents", "documents"),
                ],
                max_length=250,
            ),
        ),
        migrations.AlterField(
            model_name="historicalanalyticstable",
            name="starting_object",
            field=models.CharField(
                choices=[
                    ("cases", "Cases"),
                    ("work_items", "Work Items"),
                    ("documents", "documents"),
                ],
                max_length=250,
            ),
        ),
    ]
