# Generated by Django 3.2.13 on 2022-04-27 09:04

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("caluma_workflow", "0029_task_continue_async"),
    ]

    operations = [
        migrations.AlterField(
            model_name="case",
            name="status",
            field=models.CharField(
                choices=[
                    ("running", "Case is running and work items need to be completed."),
                    ("completed", "Case is done."),
                    ("canceled", "Case is canceled."),
                    ("suspended", "Case is suspended."),
                ],
                db_index=True,
                max_length=50,
            ),
        ),
        migrations.AlterField(
            model_name="historicalcase",
            name="status",
            field=models.CharField(
                choices=[
                    ("running", "Case is running and work items need to be completed."),
                    ("completed", "Case is done."),
                    ("canceled", "Case is canceled."),
                    ("suspended", "Case is suspended."),
                ],
                db_index=True,
                max_length=50,
            ),
        ),
        migrations.AlterField(
            model_name="historicaltask",
            name="type",
            field=models.CharField(
                choices=[
                    ("simple", "Task which can simply be marked as completed."),
                    (
                        "complete_workflow_form",
                        "Task to complete a defined workflow form.",
                    ),
                    ("complete_task_form", "Task to complete a defined task form."),
                ],
                max_length=50,
            ),
        ),
        migrations.AlterField(
            model_name="historicalworkitem",
            name="status",
            field=models.CharField(
                choices=[
                    ("ready", "Work item is ready to be processed."),
                    ("completed", "Work item is done."),
                    ("canceled", "Work item is canceled."),
                    ("skipped", "Work item is skipped."),
                    ("suspended", "Work item is suspended."),
                    ("redo", "Work item has been marked for redo."),
                ],
                db_index=True,
                max_length=50,
            ),
        ),
        migrations.AlterField(
            model_name="task",
            name="type",
            field=models.CharField(
                choices=[
                    ("simple", "Task which can simply be marked as completed."),
                    (
                        "complete_workflow_form",
                        "Task to complete a defined workflow form.",
                    ),
                    ("complete_task_form", "Task to complete a defined task form."),
                ],
                max_length=50,
            ),
        ),
        migrations.AlterField(
            model_name="workitem",
            name="status",
            field=models.CharField(
                choices=[
                    ("ready", "Work item is ready to be processed."),
                    ("completed", "Work item is done."),
                    ("canceled", "Work item is canceled."),
                    ("skipped", "Work item is skipped."),
                    ("suspended", "Work item is suspended."),
                    ("redo", "Work item has been marked for redo."),
                ],
                db_index=True,
                max_length=50,
            ),
        ),
    ]
