# Generated by Django 3.2.13 on 2022-06-09 11:26

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("caluma_workflow", "0030_remove_choices_char_field"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="historicalcase",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical case",
                "verbose_name_plural": "historical cases",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalflow",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical flow",
                "verbose_name_plural": "historical flows",
            },
        ),
        migrations.AlterModelOptions(
            name="historicaltask",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical task",
                "verbose_name_plural": "historical tasks",
            },
        ),
        migrations.AlterModelOptions(
            name="historicaltaskflow",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical task flow",
                "verbose_name_plural": "historical task flows",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalworkflow",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical workflow",
                "verbose_name_plural": "historical workflows",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalworkitem",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical work item",
                "verbose_name_plural": "historical work items",
            },
        ),
        migrations.AlterField(
            model_name="historicalcase",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicalflow",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicaltask",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicaltaskflow",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicalworkflow",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicalworkitem",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
    ]
