# -*- coding: utf-8 -*-
# Generated by Django 1.11.20 on 2019-04-11 06:07
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("caluma_form", "0010_auto_20190404_0652")]

    operations = [
        migrations.AddField(
            model_name="answer",
            name="date",
            field=models.DateField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name="answer",
            name="value",
            field=models.JSONField(blank=True, null=True),
        ),
    ]
