# Generated by Django 2.2.16 on 2020-09-18 15:59

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="AccessLog",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("timestamp", models.DateTimeField(auto_now_add=True)),
                ("username", models.CharField(blank=True, max_length=100, null=True)),
                ("query", models.TextField()),
                (
                    "variables",
                    models.JSONField(blank=True, null=True),
                ),
                (
                    "operation_name",
                    models.CharField(blank=True, max_length=100, null=True),
                ),
                ("status_code", models.PositiveIntegerField()),
                ("has_error", models.BooleanField(default=False)),
                (
                    "operation",
                    models.CharField(
                        choices=[("query", "query"), ("mutation", "mutation")],
                        max_length=20,
                    ),
                ),
                ("selection", models.CharField(blank=True, max_length=100, null=True)),
            ],
        )
    ]
